@@ -22,7 +22,8 @@ let%expect_test "Simple [of_eager] test" =
   print_s [%sexp (Deferred.peek result2 : test)];
   [%expect {|
     (Ok foo)
-    (Ok foo) |}];
+    (Ok foo)
+    |}];
   return ()
 ;;
 
@@ -38,7 +39,8 @@ let%expect_test "Simple [of_lazy] test" =
   print_s [%sexp (Deferred.peek result2 : test)];
   [%expect {|
     (Ok foo)
-    (Ok foo) |}];
+    (Ok foo)
+    |}];
   return ()
 ;;
 
@@ -99,10 +101,10 @@ let lazy_encounters_test ~run =
   let (_ : _) = run t in
   let%bind () = Scheduler.yield_until_no_jobs_remain () in
   print_s [%message "*** Forcing [y] ***"];
-  Ivar.fill force_y ();
+  Ivar.fill_exn force_y ();
   let%bind () = Scheduler.yield_until_no_jobs_remain () in
   print_s [%message "*** Forcing [prod] ***"];
-  Ivar.fill force_prod ();
+  Ivar.fill_exn force_prod ();
   Scheduler.yield_until_no_jobs_remain ()
 ;;
 
@@ -117,7 +119,8 @@ let%expect_test "[weak_run] stops when it encounters unforced lazy values" =
     (sum 3)
     "*** Forcing [prod] ***"
     (prod 2)
-    Done! |}]
+    Done!
+    |}]
 ;;
 
 let%expect_test "[force] forces lazy values it encounters" =
@@ -131,7 +134,8 @@ let%expect_test "[force] forces lazy values it encounters" =
     (prod 2)
     Done!
     "*** Forcing [y] ***"
-    "*** Forcing [prod] ***" |}]
+    "*** Forcing [prod] ***"
+    |}]
 ;;
 
 (* This test ensures that the type of run in progress isn't cached. *)
@@ -169,7 +173,8 @@ let%expect_test "When binding, if [f] raises, the error is joined" =
   [%expect
     {|
     (Error
-     (Laziness_preserving_deferred_test__Test_laziness_preserving_deferred.E_for_test)) |}];
+     (Laziness_preserving_deferred_test__Test_laziness_preserving_deferred.E_for_test))
+    |}];
   return ()
 ;;
 
@@ -185,7 +190,8 @@ let%expect_test "When binding, if [f] returns a deferred that raises, the error
   [%expect
     {|
     (Error
-     (Laziness_preserving_deferred_test__Test_laziness_preserving_deferred.E_for_test)) |}];
+     (Laziness_preserving_deferred_test__Test_laziness_preserving_deferred.E_for_test))
+    |}];
   return ()
 ;;
 
@@ -302,8 +308,8 @@ module Monad_laws_test = struct
               ~law:"Associativity"
               ~cases:
                 ([%message
-                  "" ~m:(m_case : Case.t) ~f:(f_case : Case.t) ~g:(g_case : Case.t)]
-                 : Sexp.t)
+                   "" ~m:(m_case : Case.t) ~f:(f_case : Case.t) ~g:(g_case : Case.t)]
+                  : Sexp.t)
               (m >>= f >>= g : result1)
               (m >>= fun x -> f x >>= g : result2)]
     in
