@@ -7,13 +7,7 @@ module Async = struct
   include Async_invariant_intf.Async
 
   let invariant here t sexp_of_t f =
-    match%map
-      Monitor.try_with
-        ~run:`Schedule
-        ~rest:`Log
-        f
-        ~extract_exn:true
-    with
+    match%map Monitor.try_with ~run:`Schedule ~rest:`Log f ~extract_exn:true with
     | Ok () -> ()
     | Error exn ->
       raise_s
@@ -24,11 +18,8 @@ module Async = struct
   let check_field t f wait_for_previous field =
     let%bind () = wait_for_previous in
     match%map
-      Monitor.try_with
-        ~run:`Schedule
-        ~rest:`Log
-        ~extract_exn:true
-        (fun () -> f (Field.get field t))
+      Monitor.try_with ~run:`Schedule ~rest:`Log ~extract_exn:true (fun () ->
+        f (Field.get field t))
     with
     | Ok () -> ()
     | Error exn ->
