@@ -16,10 +16,13 @@ module Flushed_result = struct
   let equal = [%compare.equal: t]
 
   let combine (l : t Deferred.t list) =
-    let%map l = Deferred.all l in
-    match List.mem l `Reader_closed ~equal with
-    | true -> `Reader_closed
-    | false -> `Ok
+    match l with
+    | [ x ] -> x
+    | l ->
+      let%map l = Deferred.all l in
+      (match List.mem l `Reader_closed ~equal with
+       | true -> `Reader_closed
+       | false -> `Ok)
   ;;
 end
 
@@ -67,7 +70,7 @@ end = struct
          passed downstream have been flushed all the way down the chain of pipes. *)
       downstream_flushed : unit -> Flushed_result.t Deferred.t
     }
-  [@@deriving fields, sexp_of]
+  [@@deriving fields ~getters ~iterators:iter, sexp_of]
 
   let invariant t : unit =
     try
@@ -76,8 +79,8 @@ end = struct
         ~pipe_id:ignore
         ~values_read:
           (check (function
-             | `Have_been_sent_downstream -> ()
-             | `Have_not_been_sent_downstream ivar -> assert (Ivar.is_empty ivar)))
+            | `Have_been_sent_downstream -> ()
+            | `Have_not_been_sent_downstream ivar -> assert (Ivar.is_empty ivar)))
         ~downstream_flushed:ignore
     with
     | exn -> raise_s [%message "Pipe.Consumer.invariant failed" (exn : exn) ~pipe:(t : t)]
@@ -98,7 +101,7 @@ end = struct
     match t.values_read with
     | `Have_been_sent_downstream -> ()
     | `Have_not_been_sent_downstream ivar ->
-      Ivar.fill ivar ();
+      Ivar.fill_exn ivar ();
       t.values_read <- `Have_been_sent_downstream
   ;;
 
@@ -129,7 +132,7 @@ module Blocked_read = struct
     { wants : 'a wants
     ; consumer : Consumer.t option
     }
-  [@@deriving fields, sexp_of]
+  [@@deriving fields ~iterators:iter, sexp_of]
 
   let invariant t : unit =
     try
@@ -137,12 +140,12 @@ module Blocked_read = struct
       Fields.iter
         ~wants:
           (check (function
-             | Zero _ | One _ -> ()
-             | At_most (i, _) -> assert (i > 0)))
+            | Zero _ | One _ -> ()
+            | At_most (i, _) -> assert (i > 0)))
         ~consumer:
           (check (function
-             | None -> ()
-             | Some consumer -> Consumer.invariant consumer))
+            | None -> ()
+            | Some consumer -> Consumer.invariant consumer))
     with
     | exn ->
       raise_s [%message "Pipe.Blocked_read.invariant failed" (exn : exn) ~pipe:(t : _ t)]
@@ -159,9 +162,9 @@ module Blocked_read = struct
 
   let fill_with_eof t =
     match t.wants with
-    | Zero i -> Ivar.fill i `Eof
-    | One i -> Ivar.fill i `Eof
-    | At_most (_, i) -> Ivar.fill i `Eof
+    | Zero i -> Ivar.fill_exn i `Eof
+    | One i -> Ivar.fill_exn i `Eof
+    | At_most (_, i) -> Ivar.fill_exn i `Eof
   ;;
 end
 
@@ -179,9 +182,9 @@ module Blocked_flush = struct
     { fill_when_num_values_read : int
     ; ready : [ `Ok | `Reader_closed ] Ivar.t
     }
-  [@@deriving fields, sexp_of]
+  [@@deriving fields ~getters, sexp_of]
 
-  let fill t v = Ivar.fill t.ready v
+  let fill t v = Ivar.fill_exn t.ready v
 end
 
 type ('a, 'phantom) t =
@@ -213,7 +216,7 @@ type ('a, 'phantom) t =
        need to write 2^62 elements to the pipe, which would take about 146 years, at a
        flow rate of 1 size-unit/nanosecond. *)
     mutable num_values_read : int
-  (* [blocked_flushes] holds flushes whose preceding elements have not been completely
+      (* [blocked_flushes] holds flushes whose preceding elements have not been completely
      read.  For each blocked flush, the number of elements that need to be read from the
      pipe in order to fill the flush is                        :
 
@@ -238,7 +241,7 @@ type ('a, 'phantom) t =
        below. *)
     upstream_flusheds : (unit -> Flushed_result.t Deferred.t) Bag.t
   }
-[@@deriving fields, sexp_of]
+[@@deriving fields ~getters ~setters ~iterators:iter, sexp_of]
 
 let effective_size_budget t = t.size_budget - t.reserved_space
 
@@ -253,6 +256,7 @@ let closed t = Ivar.read t.closed
 let pushback t = Ivar.read t.pushback
 let length t = Queue.length t.buffer
 let is_empty t = length t = 0
+let num_values_written t = length t + num_values_read t
 let update_num_values_read t ~delta = t.num_values_read <- t.num_values_read + delta
 
 let invariant t : unit =
@@ -327,8 +331,7 @@ let create_internal ~size_budget ~info ~initial_buffer =
     ; info
     ; closed = Ivar.create ()
     ; read_closed = Ivar.create ()
-    ;
-      size_budget
+    ; size_budget
     ; reserved_space = 0
     ; pushback = Ivar.create ()
     ; buffer = initial_buffer
@@ -356,7 +359,7 @@ let create ?size_budget ?info () =
   in
   let t = create_internal ~size_budget ~info ~initial_buffer:(Queue.create ()) in
   (* initially, the pipe does not pushback *)
-  Ivar.fill t.pushback ();
+  Ivar.fill_exn t.pushback ();
   if !check_invariant then invariant t;
   t, t
 ;;
@@ -373,7 +376,7 @@ let close t =
   if !check_invariant then invariant t;
   if not (is_closed t)
   then (
-    Ivar.fill t.closed ();
+    Ivar.fill_exn t.closed ();
     if is_empty t
     then (
       Queue.iter t.blocked_reads ~f:Blocked_read.fill_with_eof;
@@ -386,7 +389,7 @@ let close_read t =
   if !check_invariant then invariant t;
   if not (is_read_closed t)
   then (
-    Ivar.fill t.read_closed ();
+    Ivar.fill_exn t.read_closed ();
     Queue.iter t.blocked_flushes ~f:(fun flush -> Blocked_flush.fill flush `Reader_closed);
     Queue.clear t.blocked_flushes;
     Queue.clear t.buffer;
@@ -427,24 +430,29 @@ let create_writer ?size_budget f =
   w
 ;;
 
+let consumed_values_sent_downstream_and_flushed t =
+  if List.is_empty t.consumers
+  then return `Ok
+  else
+    Flushed_result.combine
+      (List.map t.consumers ~f:Consumer.values_sent_downstream_and_flushed)
+;;
+
+let eager_upon d f =
+  match Deferred.peek d with
+  | Some v -> f v
+  | None -> upon d f
+;;
+
 let values_were_read t consumer =
   Option.iter consumer ~f:Consumer.start;
-  let rec loop () =
-    match Queue.peek t.blocked_flushes with
-    | None -> ()
-    | Some flush ->
-      if t.num_values_read >= flush.fill_when_num_values_read
-      then (
-        ignore (Queue.dequeue_exn t.blocked_flushes : Blocked_flush.t);
-        (match consumer with
-         | None -> Blocked_flush.fill flush `Ok
-         | Some consumer ->
-           upon
-             (Consumer.values_sent_downstream_and_flushed consumer)
-             (fun flush_result -> Blocked_flush.fill flush flush_result));
-        loop ())
-  in
-  loop ()
+  let values_flushed = lazy (consumed_values_sent_downstream_and_flushed t) in
+  Queue.drain
+    t.blocked_flushes
+    ~while_:(fun flush -> t.num_values_read >= flush.fill_when_num_values_read)
+    ~f:(fun flush ->
+      eager_upon (force values_flushed) (fun flush_result ->
+        Blocked_flush.fill flush flush_result))
 ;;
 
 (* [consume_all t] reads all the elements in [t]. *)
@@ -500,10 +508,10 @@ let fill_blocked_reads t =
     let blocked_read = Queue.dequeue_exn t.blocked_reads in
     let consumer = blocked_read.consumer in
     match blocked_read.wants with
-    | Zero ivar -> Ivar.fill ivar `Ok
-    | One ivar -> Ivar.fill ivar (`Ok (consume_one t consumer))
+    | Zero ivar -> Ivar.fill_exn ivar `Ok
+    | One ivar -> Ivar.fill_exn ivar (`Ok (consume_one t consumer))
     | At_most (max_queue_length, ivar) ->
-      Ivar.fill ivar (`Ok (consume t ~max_queue_length consumer))
+      Ivar.fill_exn ivar (`Ok (consume t ~max_queue_length consumer))
   done
 ;;
 
@@ -609,6 +617,14 @@ let read_now' ?consumer ?max_queue_length t =
 ;;
 
 let read_now ?consumer t = gen_read_now t ?consumer consume_one
+
+let read_now_exn ?consumer t =
+  match read_now ?consumer t with
+  | `Ok a -> a
+  | `Eof -> raise_s [%message "Pipe.read_now_exn: received EOF"]
+  | `Nothing_available -> raise_s [%message "Pipe.read_now_exn: nothing available"]
+;;
+
 let peek t = Queue.peek t.buffer
 
 let clear t =
@@ -691,11 +707,11 @@ let read_exactly ?consumer t ~num_values =
       let already_read = Queue.length result in
       assert (already_read <= num_values);
       if already_read = num_values
-      then Ivar.fill finish (`Exactly result)
+      then Ivar.fill_exn finish (`Exactly result)
       else
         read' ?consumer t ~max_queue_length:(num_values - already_read)
         >>> function
-        | `Eof -> Ivar.fill finish (if already_read = 0 then `Eof else `Fewer result)
+        | `Eof -> Ivar.fill_exn finish (if already_read = 0 then `Eof else `Fewer result)
         | `Ok q ->
           Queue.blit_transfer ~src:q ~dst:result ();
           loop ()
@@ -705,12 +721,7 @@ let read_exactly ?consumer t ~num_values =
 
 let downstream_flushed t =
   if is_empty t
-  then
-    if List.is_empty t.consumers
-    then return `Ok
-    else
-      Flushed_result.combine
-        (List.map t.consumers ~f:Consumer.values_sent_downstream_and_flushed)
+  then consumed_values_sent_downstream_and_flushed t
   else
     (* [t] might be closed.  But the read end can't be closed, because if it were, then
        [t] would be empty.  If the write end is closed but not the read end, then we want
@@ -787,11 +798,11 @@ module Flushed = struct
 end
 
 let fold_gen
-      (read_now : ?consumer:Consumer.t -> _ Reader.t -> _)
-      ?(flushed = Flushed.When_value_read)
-      t
-      ~init
-      ~f
+  (read_now : ?consumer:Consumer.t -> _ Reader.t -> _)
+  ?(flushed = Flushed.When_value_read)
+  t
+  ~init
+  ~f
   =
   let consumer =
     match flushed with
@@ -815,7 +826,7 @@ let fold_gen
     >>> fun (_ : [ `Ok | `Eof ]) ->
     let rec loop b =
       match read_now t ?consumer with
-      | `Eof -> Ivar.fill finished b
+      | `Eof -> Ivar.fill_exn finished b
       | `Ok v -> f b v continue
       | `Nothing_available -> values_available t >>> fun _ -> loop b
     and continue b =
@@ -850,12 +861,7 @@ let with_error_to_current_monitor ?(continue_on_error = false) f a =
   if not continue_on_error
   then f a
   else (
-    match%map
-      Monitor.try_with
-        ~run:`Schedule
-        ~rest:`Log
-        (fun () -> f a)
-    with
+    match%map Monitor.try_with ~run:`Schedule ~rest:`Log (fun () -> f a) with
     | Ok () -> ()
     | Error exn -> Monitor.send_exn (Monitor.current ()) (Monitor.extract_exn exn))
 ;;
@@ -874,11 +880,11 @@ let iter ?continue_on_error ?flushed t ~f =
    rather than via [iter].  The implementation reads only one element at a time, so that
    if [f] closes [t] or raises, no more elements will be read. *)
 let iter_without_pushback
-      ?consumer
-      ?(continue_on_error = false)
-      ?max_iterations_per_job
-      t
-      ~f
+  ?consumer
+  ?(continue_on_error = false)
+  ?max_iterations_per_job
+  t
+  ~f
   =
   ensure_consumer_matches t ?consumer;
   let max_iterations_per_job =
@@ -898,8 +904,8 @@ let iter_without_pushback
     then f
     else
       fun a ->
-        try f a with
-        | exn -> Monitor.send_exn (Monitor.current ()) exn
+      try f a with
+      | exn -> Monitor.send_exn (Monitor.current ()) exn
   in
   Deferred.create (fun finished ->
     (* We do [values_available t >>>] to ensure that [f] is only called asynchronously.
@@ -912,7 +918,7 @@ let iter_without_pushback
       then return () >>> fun () -> start ()
       else (
         match read_now t ?consumer with
-        | `Eof -> Ivar.fill finished ()
+        | `Eof -> Ivar.fill_exn finished ()
         | `Ok a ->
           f a;
           loop ~remaining:(remaining - 1)
@@ -990,11 +996,11 @@ let of_stream_deprecated s =
 ;;
 
 let transfer_gen
-      (read_now : ?consumer:Consumer.t -> _ Reader.t -> _)
-      write
-      input
-      output
-      ~f
+  (read_now : ?consumer:Consumer.t -> _ Reader.t -> _)
+  write
+  input
+  output
+  ~f
   =
   if !check_invariant
   then (
@@ -1017,7 +1023,7 @@ let transfer_gen
     let output_closed () =
       close_read input;
       unlink ();
-      Ivar.fill result ()
+      Ivar.fill_exn result ()
     in
     let rec loop () =
       if is_closed output
@@ -1026,7 +1032,7 @@ let transfer_gen
         match read_now input ~consumer with
         | `Eof ->
           unlink ();
-          Ivar.fill result ()
+          Ivar.fill_exn result ()
         | `Ok x -> f x continue
         | `Nothing_available -> input_available_or_output_closed () >>> fun () -> loop ())
     and continue y =
@@ -1106,7 +1112,7 @@ let filter input ~f = filter_map input ~f:(fun x -> if f x then Some x else None
 
 let of_queue_internal queue =
   let t = create_internal ~size_budget:0 ~info:None ~initial_buffer:queue in
-  Ivar.fill t.closed ();
+  Ivar.fill_exn t.closed ();
   update_pushback t;
   t
 ;;
@@ -1185,15 +1191,22 @@ let to_sequence t =
     | `Nothing_available -> Some (Wait_for (values_available t), ()))
 ;;
 
-let interleave_pipe inputs =
+let interleave_pipe ?(close_on = `All_inputs_closed) inputs =
   let output, output_writer = create ~info:[%sexp "Pipe.interleave"] () in
   (* We keep a reference count of all the pipes that [interleave_pipe] is managing;
      [inputs] counts as one.  When the reference count drops to zero, we know that all
      pipes are closed and we can close [output_writer]. *)
   let num_pipes_remaining = ref 1 in
-  let decr_num_pipes_remaining () =
+  let close_one_pipe closed_pipe_kind =
     decr num_pipes_remaining;
-    if !num_pipes_remaining = 0 then close output_writer
+    let should_close =
+      match close_on, closed_pipe_kind with
+      | `Any_input_closed, `Input_pipe -> true
+      | `Any_input_closed, `Pipe_of_input_pipes
+      | `All_inputs_closed, (`Input_pipe | `Pipe_of_input_pipes) ->
+        !num_pipes_remaining = 0
+    in
+    if should_close then close output_writer
   in
   don't_wait_for
     (let%map () =
@@ -1201,16 +1214,16 @@ let interleave_pipe inputs =
          incr num_pipes_remaining;
          don't_wait_for
            (let%map () = transfer_id input output_writer in
-            decr_num_pipes_remaining ()))
+            close_one_pipe `Input_pipe))
      in
-     decr_num_pipes_remaining ());
+     close_one_pipe `Pipe_of_input_pipes);
   (* for [inputs] *)
   output
 ;;
 
-let interleave inputs =
+let interleave ?close_on inputs =
   if !check_invariant then List.iter inputs ~f:invariant;
-  interleave_pipe (of_list inputs)
+  interleave_pipe ?close_on (of_list inputs)
 ;;
 
 let merge inputs ~compare =
