@@ -36,7 +36,6 @@ module Writer : sig
   type phantom
   type 'a t = ('a, phantom) pipe [@@deriving sexp_of]
 
-
   val invariant : _ t -> unit
 end
 
@@ -44,7 +43,6 @@ module Reader : sig
   type phantom
   type 'a t = ('a, phantom) pipe [@@deriving sexp_of]
 
-
   val invariant : _ t -> unit
 end
 
@@ -163,7 +161,7 @@ module Flushed_result : sig
     [ `Ok
     | `Reader_closed
     ]
-  [@@deriving compare, sexp_of]
+  [@@deriving compare, equal, sexp_of]
 end
 
 (** Deferreds returned by [upstream_flushed] and [downstream_flushed] become determined
@@ -243,10 +241,11 @@ module Consumer : sig
 end
 
 (** [add_consumer reader ~downstream_flushed] creates a new consumer of [reader], and
-    causes future calls to [flushed_downstream reader] to take this consumer into account.
-    Thereafter, [Pipe.flushed_downstream reader] will first ensure that values previously
-    written to [reader] have been read, then that they have been sent downstream by the
-    consumer that read them, and finally that they have been flushed downstream.
+    causes future calls to [Pipe.downstream_flushed reader] to take this consumer into
+    account. Thereafter, [Pipe.downstream_flushed reader] will first ensure that values
+    previously written to [reader] have been read, then that they have been sent
+    downstream by the consumer that read them, and finally that they have been flushed
+    downstream.
 
     One should only supply the resulting consumer to read operations on [reader].  Using
     a consumer created from one reader with another reader will raise an exception. *)
@@ -266,6 +265,24 @@ val length : (_, _) t -> int
 (** [is_empty t] is true iff there are no values in the pipe. *)
 val is_empty : (_, _) t -> bool
 
+(** [num_values_read] keeps track of the total number of values that have been read from
+    the pipe.
+
+    It can only continue to increase as long as the read end of the pipe remains open.
+    It can never decrease.
+
+    At any given time, [num_values_read = num_values_written - length]. *)
+val num_values_read : (_, _) t -> int
+
+(** [num_values_written] keeps track of the total number of values that have been written
+    to the pipe.
+
+    It can only continue to increase as long as the write end of the pipe remains open.
+    It can never decrease.
+
+    At any given time, [num_values_written = num_values_read + length]. *)
+val num_values_written : (_, _) t -> int
+
 (** {2 Writing} *)
 
 (** The write operations return a deferred value that is determined when either (1) it is
@@ -388,7 +405,6 @@ val read : ?consumer:Consumer.t -> 'a Reader.t -> [ `Eof | `Ok of 'a ] Deferred.
 (** [read_exn] is like [read], except it raises on [`Eof]. *)
 val read_exn : ?consumer:Consumer.t -> 'a Reader.t -> 'a Deferred.t
 
-
 (** [read_exactly r ~num_values] reads exactly [num_values] items, unless EOF is
     encountered. [read_exactly] performs a sequence of [read'] operations, so there is no
     guarantee that the queue of values it returns comprise a contiguous segment of the
@@ -404,11 +420,11 @@ val read_exactly
      | `Fewer of 'a Queue.t (** [0 < Q.length q < num_values] *)
      | `Exactly of 'a Queue.t (** [Q.length q = num_values] *)
      ]
-       Deferred.t
+     Deferred.t
 
 (** [read_now' reader] reads values from [reader] that are immediately available.  If
-    [reader] is closed, [read_now'] returns [`Eof].  If [reader] is empty, [read_now']
-    returns [`Nothing_available].  Otherwise, [`Ok q] is returned, and the resulting queue
+    [reader] is empty, [read_now'] returns [`Eof] if [reader] is closed and
+    [`Nothing_available] if not.  Otherwise, [`Ok q] is returned, and the resulting queue
     will satisfy [0 < Q.length q <= max_queue_length].  The [consumer] is used to extend
     the meaning of values being flushed (see the [Consumer] module above). *)
 val read_now'
@@ -424,6 +440,9 @@ val read_now
   -> 'a Reader.t
   -> [ `Eof | `Nothing_available | `Ok of 'a ]
 
+(** [read_now_exn] is like [read_now], but it raises on the [`Eof] and
+    [`Nothing_available] cases. *)
+val read_now_exn : ?consumer:Consumer.t -> 'a Reader.t -> 'a
 
 val peek : 'a Reader.t -> 'a option
 
@@ -435,7 +454,6 @@ val clear : 'a Reader.t -> unit
     alternative name might be [Reader.to_queue]. *)
 val read_all : 'a Reader.t -> 'a Queue.t Deferred.t
 
-
 (** [values_available reader] returns a deferred that becomes determined when there are
     values in the pipe.  If there are multiple readers (a rare situation), there is no
     guarantee that some other reader hasn't become active because of ordinary Async
@@ -481,7 +499,6 @@ val read_choice_single_consumer_exn
 (** {2 Sequence functions} *)
 
 module Flushed : sig
-
   type t =
     | Consumer of Consumer.t
     | When_value_processed
@@ -527,7 +544,6 @@ val fold'
   -> f:('accum -> 'a Queue.t -> 'accum Deferred.t)
   -> 'accum Deferred.t
 
-
 (** [fold reader ~init ~f] folds over the elements of [reader], consuming them as they
     come in.  [fold] finishes when the final call to [f] returns. *)
 
@@ -582,7 +598,6 @@ val iter_without_pushback
   -> f:('a -> unit)
   -> unit Deferred.t
 
-
 (** [transfer' input output ~f] repeatedly reads a batch of elements from [input], applies
     [f] to the batch, writes the result as a batch to [output], and then waits on
     [pushback] in [output] before continuing.  [transfer'] finishes if [input] is closed
@@ -686,12 +701,25 @@ val filter : 'a Reader.t -> f:('a -> bool) -> 'a Reader.t
     of values from that input to [output], using [transfer_id].  Each input is transferred
     to [output] independently.  So, batches of values from different inputs can be in
     flight to [output] simultaneously, but at most one batch at a time from any particular
-    input.  The operation is complete when either all the [inputs] produce EOF, or when
-    [output] is closed by the downstream consumer (in which case [interleave] closes all
-    the [inputs]). *)
-val interleave : 'a Reader.t list -> 'a Reader.t
+    input.
 
-val interleave_pipe : 'a Reader.t Reader.t -> 'a Reader.t
+    When [output] is closed by the downstream consumer, [interleave] closes
+    all the [inputs].
+
+    [close_on] defines the conditions under which the output pipe will close:
+    - [`All_inputs_closed] will only close the output if all inputs have closed
+    - [`Any_input_closed] closes the output once any input has closed *)
+val interleave
+  :  ?close_on:[ `All_inputs_closed | `Any_input_closed ]
+       (** default: [`All_inputs_closed] *)
+  -> 'a Reader.t list
+  -> 'a Reader.t
+
+val interleave_pipe
+  :  ?close_on:[ `All_inputs_closed | `Any_input_closed ]
+       (** default: [`All_inputs_closed] *)
+  -> 'a Reader.t Reader.t
+  -> 'a Reader.t
 
 (** [merge inputs ~compare] returns a reader, [output], that merges all the inputs.
     Assuming that for each input, values are sorted according to the comparison function
@@ -722,7 +750,9 @@ val concat_pipe : 'a Reader.t Reader.t -> 'a Reader.t
     faster one, but [fork] could be forced to buffer arbitrarily many elements.  Beware of
     unbounded resource usage in downstream code where one reader might fall behind.
 
-    Note that {!upstream_flushed} will not work with the pipes returned by [fork]. *)
+    Note that {!downstream_flushed} will only mark elements flushed once they've been
+    flushed by both returned pipes, regardless of the value used for [pushback_uses].
+    Additionally, {!upstream_flushed} will not work with the pipes returned by [fork]. *)
 val fork
   :  'a Reader.t
   -> pushback_uses:[ `Both_consumers | `Fast_consumer_only ]
