@@ -19,17 +19,17 @@ module T1 = struct
   (* We don't include the [id] in the sexp because the user (rightly) can't control it, so
      it's hard to make it deterministic in tests. *)
   let sexp_of_t
-        _
-        { id = _
-        ; advance_errors = _
-        ; am_advancing = _
-        ; events
-        ; fired_events = _
-        ; handle_fired = _
-        ; is_wall_clock
-        ; most_recently_fired = _
-        ; scheduler = _
-        }
+    _
+    { id = _
+    ; advance_errors = _
+    ; am_advancing = _
+    ; events
+    ; fired_events = _
+    ; handle_fired = _
+    ; is_wall_clock
+    ; most_recently_fired = _
+    ; scheduler = _
+    }
     =
     if is_wall_clock
     then [%message "<wall_clock>"]
@@ -82,6 +82,11 @@ let now t =
 (* We preallocate [send_exn] to avoid allocating it on each call to [advance_clock]. *)
 let send_exn = Some Monitor.send_exn
 let advance_directly t ~to_ = Synchronous_time_source0.advance_clock t ~to_ ~send_exn
+
+let advance_directly_stop_on_next_alarm t ~to_ =
+  Synchronous_time_source0.advance_clock_stop_at_next_alarm t ~to_ ~send_exn
+;;
+
 let advance_directly_by t by = advance_directly t ~to_:(Time_ns.after (now t) by)
 let advance = advance_directly
 let advance_by = advance_directly_by
@@ -119,21 +124,16 @@ let advance_by_alarms ?wait_for t ~to_ =
   let one_step () =
     if Synchronous_time_source0.any_fired_events_to_run t
     then now t, `continue
-    else (
-      match Timing_wheel.min_alarm_time_in_min_interval t.events with
-      | None -> to_, `finish
-      | Some min_alarm_time_in_min_interval ->
-        if Time_ns.( >= ) min_alarm_time_in_min_interval to_
-        then to_, `finish
-        else min_alarm_time_in_min_interval, `continue)
+    else to_, `check_time
   in
   let rec walk_alarms () =
     let advance_to, next = one_step () in
-    advance_directly t ~to_:advance_to;
+    advance_directly_stop_on_next_alarm t ~to_:advance_to;
     fire_past_alarms t;
     Eager_deferred.bind_unit (run_queued_alarms ()) ~f:(fun () ->
       match next with
-      | `finish -> return ()
+      | `check_time ->
+        if Time_ns.( < ) (timing_wheel_now t) to_ then walk_alarms () else return ()
       | `continue -> walk_alarms ())
   in
   (* This first [run_queued_alarms] call allows [Clock_ns.every] the opportunity to run
@@ -214,7 +214,7 @@ let run_at t time f a = ignore (run_at_internal t time f a : _ Alarm.t)
 let run_after t span f a = run_at t (span_to_time t span) f a
 
 let at =
-  let fill result = Ivar.fill result () in
+  let fill result = Ivar.fill_exn result () in
   fun t time ->
     if Time_ns.( <= ) time (Timing_wheel.now t.events)
     then return ()
@@ -271,7 +271,7 @@ module Event = struct
       mutable scheduled_at : Time_ns.t
     ; time_source : Synchronous_time_source0.t
     }
-  [@@deriving fields, sexp_of]
+  [@@deriving fields ~getters ~iterators:iter, sexp_of]
 
   type t_unit = (unit, unit) t [@@deriving sexp_of]
 
@@ -333,7 +333,7 @@ module Event = struct
     | Some (Aborted a) -> Previously_aborted a
     | Some (Happened h) -> Previously_happened h
     | None ->
-      Ivar.fill t.fired (Aborted a);
+      Ivar.fill_exn t.fired (Aborted a);
       remove_alarm_if_scheduled t.time_source t.alarm;
       Ok
   ;;
@@ -409,7 +409,7 @@ module Event = struct
         else (
           let result = f z in
           (* [f z] may have aborted the event, so we must check [fired] again. *)
-          if Ivar.is_empty t.fired then Ivar.fill t.fired (Happened result))
+          if Ivar.is_empty t.fired then Ivar.fill_exn t.fired (Happened result))
     in
     t.fire <- fire;
     schedule t;
@@ -472,13 +472,13 @@ module Continue = struct
 end
 
 let run_repeatedly
-      ?(start = return ())
-      ?stop
-      ?(continue_on_error = true)
-      ?(finished = Ivar.create ())
-      t
-      ~f
-      ~continue
+  ?(start = return ())
+  ?stop
+  ?(continue_on_error = true)
+  ?(finished = Ivar.create ())
+  t
+  ~f
+  ~continue
   =
   start
   >>> fun () ->
@@ -585,6 +585,12 @@ let with_timeout t span d =
     ]
 ;;
 
+let with_timeout_exn t span d ~error =
+  Deferred.map (with_timeout t span d) ~f:(function
+    | `Result result -> result
+    | `Timeout -> Error.raise error)
+;;
+
 let duration_of t f =
   let start = now t in
   (* Eager map to provide more accurate timings when [f] is synchronous. *)
